use anyhow::{
    Context,
    Result,
};
use feed_rs::{
    parser::parse,
    model::Feed,
};

use reqwest::IntoUrl;

fn fetch_feed(url: impl IntoUrl) -> Result<Feed> {
    let content = reqwest::blocking::get(url)?.bytes()?;
    Ok(parse(&content[..])?)
}

fn main() -> Result<()> {

    let mut feed_items = Vec::new();

    feed_items.extend(fetch_feed("https://www.youtube.com/feeds/videos.xml?channel_id=UC0intLFzLaudFG-xAvUEO-A")?.entries);
    feed_items.extend(fetch_feed("https://feed.zugfunk-podcast.de/")?.entries);

    feed_items.sort_by(|a, b| a.published.unwrap_or(chrono::DateTime::UNIX_EPOCH).cmp(&b.published.unwrap_or(chrono::DateTime::UNIX_EPOCH)).reverse());

    let mut out = String::new();

    out.push_str("<!DOCTYPE html>\n");
    out.push_str("<html>\n");
    out.push_str("<head>\n");
    out.push_str("</head>\n");
    out.push_str("<body>\n");

    out.push_str("<ul>\n");

    for item in feed_items {
        out.push_str(&format!("<li><a href=\"{}\">{}</a> ({})</li>\n", item.links.first().context("No link found")?.href, item.title.context("No title found")?.content, item.published.context("")?));
    }

    out.push_str("</ul>\n");

    out.push_str("</body>\n");
    out.push_str("</html>\n");

    std::fs::write("index.html", out)?;

    Ok(())
}
